#!/usr/bin/env python
from __future__ import division

"""compare_directory.py: compare two directories, using regexes

XXX: want to keep track of all files in new directory
"""

__version__ = "$Revision: 5583 $"

## Copyright 2011 Michael M. Hoffman <mmh1@uw.edu>

import filecmp
from os import walk
from re import compile as re_compile, escape
import sys

from path import path

from segway._util import maybe_gzip_open

def get_dir_filenames(dirname):
    for dirbasename, dirnames, filenames in walk(dirname):
        dirbasepath = path(dirbasename)
        relative_dirbasename = dirbasename.partition(dirname)[2]

        if relative_dirbasename.startswith("/"):
            relative_dirbasename = relative_dirbasename[1:]

        relative_dirpath = path(relative_dirbasename)

        try:
            dirnames.remove(".svn")
        except ValueError:
            pass

        dirnames.sort()

        for filename in sorted(filenames):
            filename_relative = str(relative_dirpath / filename)
            # not really absolute, but more so than relative
            filename_absolute = str(dirbasepath / filename)
            yield filename_relative, filename_absolute

# regular expression unescape
re_unescape = re_compile(r"\(%.*?%\)")
def make_regex(text):
    """
    make regex, escaping things that aren't with (% %)
    """
    spans = [match.span() for match in re_unescape.finditer(text)]

    res = ["^"]
    last_end = 0
    for start, end in spans:
        res.append(escape(text[last_end:start]))
        res.append(text[start+2:end-2]) # eliminate (% and %)
        last_end = end
    res.extend([escape(text[last_end:]), "$"])

    return re_compile("".join(res))

def compare_file(template_filename, query_filename):
    # quick comparison without regexes
    if filecmp.cmp(template_filename, query_filename, shallow=False):
        return True # files are identical, skip slow regex stuff

    with maybe_gzip_open(template_filename) as template_file:
        re_template = make_regex(template_file.read())

    with maybe_gzip_open(query_filename) as query_file:
        match = re_template.match(query_file.read())

    return bool(match)

class TestCounter(object):
    def __init__(self):
        self.num_error = 0
        self.num_success = 0

    def error(self, msg):
        self.num_error += 1
        print >>sys.stderr, " %s" % msg

    def success(self):
        self.num_success += 1

def compare_directory(template_dirname, query_dirname):
    counter = TestCounter()
    query_filenames = dict(get_dir_filenames(query_dirname))

    template_filenames = get_dir_filenames(template_dirname)
    for template_filename_relative, template_filename in template_filenames:
        re_template_filename_relative = make_regex(template_filename_relative)

        for query_filename_relative, query_filename in query_filenames.iteritems():
            if re_template_filename_relative.match(query_filename_relative):
                del query_filenames[query_filename_relative]
                if compare_file(template_filename, query_filename):
                    counter.success()
                else:
                    counter.error("%s and %s differ" % (template_filename, query_filename))

                break
        else:
            counter.error("query directory missing %s" % template_filename)

    for query_filename_relative in query_filenames.iterkeys():
        counter.error("template directory missing %s" % query_filename_relative)

    if counter.num_error == 0:
        print >>sys.stderr, "PASS: %s and %s: %d files match" % (template_dirname, query_dirname, counter.num_success)
        return 0
    else:
        msg = "FAIL: %s and %s: %d files match; %d files mismatch" % (template_dirname, query_dirname, counter.num_success, counter.num_error)
        print >>sys.stderr, msg
        return 1

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... TEMPLATEDIR QUERYDIR"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    options, args = parser.parse_args(args)

    if not len(args) == 2:
        parser.error("incorrect number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return compare_directory(*args)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division

__version__ = "$Revision: 5152 $"

# Copyright 2008-2011 Michael M. Hoffman <mmh1@washington.edu>

from itertools import chain
import re
import sys

FIELDNAMES = ["chrom", "chromStart", "chromEnd", # required
              "name", "score", "strand", "thickStart", "thickEnd", "itemRgb",
              "blockCount", "blockSizes", "blockStarts"]

class Datum(object):
    def __init__(self, words):
        self.__dict__ = dict(zip(FIELDNAMES, words))
        self._words = tuple(words)

    def __repr__(self):
        return "%s%s" % (self.__class__.__name__, self._words)

class NativeDatum(Datum):
    def __init__(self, *args, **kwargs):
        Datum.__init__(self, *args, **kwargs)

        # zero-based, http://genome.ucsc.edu/FAQ/FAQformat#format1
        self.chromStart = int(self.chromStart)
        self.chromEnd = int(self.chromEnd)

        try:
            self.score = float(self.score)
        except AttributeError:
            self._words = (self.chrom, self.chromStart,
                           self.chromEnd) + self._words[3:]
        else:
            self._words = (self.chrom, self.chromStart, self.chromEnd,
                           self.name, self.score) + self._words[5:]

def read(iterator, datum_cls=Datum):
    for line in iterator:
        words = line.split()
        if not words or words[0] == "track":
            continue # skip

        assert len(words) >= 3

        yield datum_cls(words)

def read_native(*args, **kwargs):
    return read(datum_cls=NativeDatum, *args, **kwargs)

re_trackline_split = re.compile(r"(?:[^ =]+=([\"'])[^\1]+?\1(?= |$)|[^ ]+)")
def get_trackline_and_reader(iterator, datum_cls=Datum):
    line = iterator.next()

    if line.startswith("track"):
        # retrieves group 1 of re_trackline_split match, which is the whole item
        trackline = [match.group(0)
                     for match in re_trackline_split.finditer(line)]
        reader = read(iterator, datum_cls)
    else:
        trackline = []
        reader = read(chain([line], iterator), datum_cls)

    return trackline, reader

def get_trackline_and_reader_native(*args, **kwargs):
    return get_trackline_and_reader(datum_cls=NativeDatum, *args, **kwargs)

def main(args=sys.argv[1:]):
    pass

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 5777 $"

# Copyright 2008-2009, 2011 Michael M. Hoffman <mmh1@washington.edu>

from collections import defaultdict
from contextlib import closing
from functools import partial
from gzip import open as _gzip_open
from itertools import repeat
from os import extsep
import shutil
import sys
import re
from tempfile import mkdtemp

import colorbrewer
from numpy import (add, append, arcsinh, array, column_stack, diff, empty,
                   insert, intc, invert, isnan, maximum, zeros)

from optbuild import Mixin_UseFullProgPath, OptionBuilder_ShortOptWithSpace_TF
from path import path
from pkg_resources import resource_filename, resource_string
from tables import Filters, NoSuchNodeError, openFile

# XXX: check that these are all in use

# these are loaded by other modules indirectly
# ignore PyFlakes warnings here

try:
    # Python 2.6
    PKG = __package__
except NameError:
    PKG = "segway"

PKG_DATA = ".".join([PKG, "data"])

FILTERS_GZIP = Filters(complevel=1)

EXT_LIST = "list"
EXT_INT = "int"
EXT_FLOAT = "float32"
EXT_GZ = "gz"
EXT_MASTER = "master"
EXT_PARAMS = "params"
EXT_TAB = "tab"

SUFFIX_GZ = extsep + EXT_GZ

DTYPE_IDENTIFY = intc
DTYPE_OBS_INT = intc
DTYPE_SEG_LEN = intc

DISTRIBUTION_NORM = "norm"
DISTRIBUTION_GAMMA = "gamma"
DISTRIBUTION_ASINH_NORMAL = "asinh_norm"

PREFIX_INPUT = "input"
PREFIX_LIKELIHOOD = "likelihood"
PREFIX_PARAMS = "params"

SUBDIRNAME_LOG = "log"
SUBDIRNAME_PARAMS = "params"

POSTERIOR_SCALE_FACTOR = 100.0

# sentinel values
ISLAND_BASE_NA = 0
ISLAND_LST_NA = 0

ORD_A = ord("A")
ORD_C = ord("C")
ORD_G = ord("G")
ORD_T = ord("T")
ORD_a = ord("a")
ORD_c = ord("c")
ORD_g = ord("g")
ORD_t = ord("t")

data_filename = partial(resource_filename, PKG_DATA)
data_string = partial(resource_string, PKG_DATA)

NUM_COLORS = 8
SCHEME = colorbrewer.Dark2[NUM_COLORS]

KB = 2**10
MB = 2**20
GB = 2**30
TB = 2**40
PB = 2**50
EB = 2**60

OptionBuilder_GMTK = (Mixin_UseFullProgPath +
                      OptionBuilder_ShortOptWithSpace_TF)

VITERBI_PROG = OptionBuilder_GMTK("gmtkViterbi")
POSTERIOR_PROG = OptionBuilder_GMTK("gmtkJT")

BED_SCORE = "1000"
BED_STRAND = "."

def extjoin(*args):
    return extsep.join(args)

# NamedTemporaryDir is based somewhat on Python 2.5.2
# tempfile._TemporaryFileWrapper
#
# Original Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006 Python
# Software Foundation; All Rights Reserved
#
# License at http://www.python.org/download/releases/2.5.2/license/

# XXX: submit for inclusion in core
# XXX: is this still in use?
class NamedTemporaryDir(object):
    def __init__(self, *args, **kwargs):
        self.name = mkdtemp(*args, **kwargs)
        self.close_called = False
        self.rmtree = shutil.rmtree # want a function, not an unbound method

    def __enter__(self):
        return self

    def close(self):
        if not self.close_called:
            self.close_called = True
            self.rmtree(self.name)

    def __del__(self):
        self.close()

    def __exit__(self, exc, value, tb):
        self.close()

def die(msg=""):
    if msg:
        print >>sys.stderr, msg
    sys.exit(1)

def get_col_index(chromosome, trackname):
    return get_tracknames(chromosome).index(trackname)

def get_label_color(label):
    color = SCHEME[label % NUM_COLORS]

    return ",".join(map(str, color))

# XXX: suggest as default
def fill_array(scalar, shape, dtype=None, *args, **kwargs):
    if dtype is None:
        dtype = array(scalar).dtype

    res = empty(shape, dtype, *args, **kwargs)
    res.fill(scalar)

    return res

# XXX: suggest as default
def gzip_open(*args, **kwargs):
    return closing(_gzip_open(*args, **kwargs))

def maybe_gzip_open(filename, mode="r", *args, **kwargs):
    if filename == "-":
        if mode.startswith("U"):
            raise NotImplementedError("U mode not implemented")
        elif mode.startswith("w") or mode.startswith("a"):
            return sys.stdout
        elif mode.startswith("r"):
            if "+" in mode:
                raise NotImplementedError("+ mode not implemented")
            else:
                return sys.stdin
        else:
            raise ValueError("mode string must begin with one of 'r', 'w', or 'a'")

    if filename.endswith(SUFFIX_GZ):
        return gzip_open(filename, mode, *args, **kwargs)

    return open(filename, mode, *args, **kwargs)

def constant(val):
    """
    constant values for defaultdict
    """
    return repeat(val).next

array_factory = constant(array([]))

# XXX: replace with genomedata.Chromosome.tracknames_continuous
def get_tracknames(chromosome):
    return chromosome.root._v_attrs.tracknames.tolist()

def init_num_obs(num_obs, continuous):
    curr_num_obs = continuous.shape[1]
    assert num_obs is None or num_obs == curr_num_obs

    return curr_num_obs

def new_extrema(func, data, extrema):
    curr_extrema = func(data, 0)

    return func([extrema, curr_extrema], 0)

# XXX: replace with iter(genomedata.Chromosome)
def walk_supercontigs(h5file):
    root = h5file.root

    for supercontig in h5file.walkGroups():
        if supercontig == root:
            continue

        yield supercontig

# XXX: replace with genomedata.Chromosome.itercontinuous
def walk_continuous_supercontigs(h5file):
    for supercontig in walk_supercontigs(h5file):
        try:
            yield supercontig, supercontig.continuous
        except NoSuchNodeError:
            continue

# XXX: duplicates bed.py?
def load_coords(filename):
    if not filename:
        return

    coords = defaultdict(list)

    with maybe_gzip_open(filename) as infile:
        for line in infile:
            words = line.rstrip().split()
            chrom = words[0]
            start = int(words[1])
            end = int(words[2])

            coords[chrom].append((start, end))

    return defaultdict(array_factory, ((chrom, array(coords_list))
                       for chrom, coords_list in coords.iteritems()))

def get_chrom_coords(coords, chrom):
    """
    returns empty array if there are no coords on that chromosome
    returns None if there are no coords whatsoever
    """
    if coords:
        return coords[chrom]

def is_empty_array(arr):
    try:
        return arr.shape == (0,)
    except AttributeError:
        return False

def chrom_name(filename):
    return path(filename).namebase

# XXX: replace with stuff from prep_observations()
def iter_chroms_coords(filenames, coords):
    for filename in filenames:
        print >>sys.stderr, filename
        chrom = chrom_name(filename)

        chr_include_coords = get_chrom_coords(coords, chrom)

        if is_empty_array(chr_include_coords):
            continue

        with openFile(filename) as chromosome:
            yield chrom, filename, chromosome, chr_include_coords

def find_segment_starts(data):
    """
    finds the start of each segment

    returns the start positions, and the labels at each position

    returns lists of len num_segments+1, num_segments
    """
    len_data = len(data)

    # unpack tuple, ignore rest
    end_pos, = diff(data).nonzero()

    # add one to get the start positions, and add a 0 at the beginning
    start_pos = insert(end_pos + 1, 0, 0)
    labels = data[start_pos]

    # after generating labels, add an extraneous start position so
    # where_seg+1 doesn't go out of bounds
    start_pos = append(start_pos, len_data)

    return start_pos, labels

def ceildiv(dividend, divisor):
    "integer ceiling division"

    # int(bool) means 0 -> 0, 1+ -> 1
    return (dividend // divisor) + int(bool(dividend % divisor))

def downsample_add(inarray, resolution):
    # downsample presence data into num_datapoints
    full_num_rows = inarray.shape[0]

    downsampled_num_rows = ceildiv(full_num_rows, resolution)
    remainder = full_num_rows % resolution
    downsampled_shape = [downsampled_num_rows] + list(inarray.shape[1:])

    res = zeros(downsampled_shape, inarray.dtype)

    # if there's no remainder, then only use loop 0
    if remainder == 0:
        remainder = resolution

    # loop 0: every bit
    for index in xrange(remainder):
        res += inarray[index::resolution]

    # loop 1: remainder
    for index in xrange(remainder, resolution):
        res[:-1] += inarray[index::resolution]

    return res

def make_dinucleotide_int_data(seq):
    """
    makes an array with two columns, one with 0..15=AA..TT and the other
    as a presence variable. Set column one to 0 when not present
    """
    nucleotide_int_data = (((seq == ORD_A) + (seq == ORD_a))
                           + ((seq == ORD_C) + (seq == ORD_c)) * 2
                           + ((seq == ORD_G) + (seq == ORD_g)) * 3
                           + ((seq == ORD_T) + (seq == ORD_t)) * 4) - 1
    nucleotide_missing = nucleotide_int_data == -1

    # rewrite all Ns as A now that you have the missingness mask
    nucleotide_int_data[nucleotide_int_data == -1] = 0
    col_shape = (len(nucleotide_int_data)-1,)

    # first column: dinucleotide: AA..TT=0..15
    # combine, and add extra AA stub at end, which will be set missing
    # 0 AA AC AG AT
    # 4 CA CC CG CT
    # 8 GA GC GG GT
    # 12 TA TC TG TT
    dinucleotide_int_data = empty(col_shape, DTYPE_OBS_INT)
    add(nucleotide_int_data[:-1] * 4, nucleotide_int_data[1:],
        dinucleotide_int_data)

    # second column: presence_dinucleotide: some missing=0; all present = 1
    # there are so few N boundaries that it is okay to
    # disregard the whole dinucleotide when half is N
    dinucleotide_missing = (nucleotide_missing[:-1] + nucleotide_missing[1:])

    dinucleotide_presence = empty(dinucleotide_missing.shape, DTYPE_OBS_INT)
    invert(dinucleotide_missing, dinucleotide_presence)

    # XXXopt: set these up properly in the first place instead of
    # column_stacking at the end
    return column_stack([dinucleotide_int_data, dinucleotide_presence])

def _save_observations_window(float_filename, int_filename, float_data,
                             resolution, distribution, seq_data=None,
                             supervision_data=None):
    # this is now a naked function so that it can be called by task.py

    # input function in GMTK_ObservationMatrix.cc:
    # ObservationMatrix::readBinSentence

    # input per frame is a series of float32s, followed by a series of
    # int32s it is better to optimize both sides here by sticking all
    # the floats in one file, and the ints in another one
    int_blocks = []
    if float_data is not None:
        if distribution == DISTRIBUTION_ASINH_NORMAL:
            float_data = arcsinh(float_data)

        mask_missing = isnan(float_data)

        # output -> presence_data -> int_blocks
        # done in two steps so I can specify output type
        presence_data = empty(mask_missing.shape, DTYPE_OBS_INT)
        invert(mask_missing, presence_data)

        num_datapoints = downsample_add(presence_data, resolution)

        int_blocks.append(num_datapoints)

        # so that there is no divide by zero
        num_datapoints_min_1 = maximum(num_datapoints, 1)

        # make float
        float_data[mask_missing] = 0.0

        float_data_downsampled = downsample_add(float_data, resolution)
        float_data_downsampled /= num_datapoints_min_1
        float_data_downsampled.tofile(float_filename)

    if seq_data is not None:
        assert resolution == 1 # not implemented yet
        int_blocks.append(make_dinucleotide_int_data(seq_data))

    if supervision_data is not None:
        assert resolution == 1 # not implemented yet
        int_blocks.append(supervision_data)

    # XXXopt: use the correctly sized matrix in the first place
    int_data = column_stack(int_blocks)
    int_data.tofile(int_filename)

# XXX: there is a new genomedata interface that should be able to
# replace the callers to this
def _make_continuous_cells(supercontig, start, end, track_indexes):
    continuous = supercontig.continuous
    if continuous is None:
        return

    # window_start: relative to the beginning of the supercontig
    window_start = start - supercontig.start
    window_end = end - supercontig.start

    # XXXopt: reading all the extra tracks is probably quite wasteful
    # given the genomedata striping pattern; it is probably better to
    # read one at a time and stick into an array
    min_col = track_indexes.min()
    max_col = track_indexes.max() + 1

    # first, extract a contiguous subset of the tracks in the dataset,
    # which is a superset of the tracks that are used
    rows = continuous[window_start:window_end, min_col:max_col]

    # extract only the tracks that are used correct for min_col offset
    return rows[..., track_indexes - min_col]

re_posterior_entry = re.compile(r"^\d+: (\S+) seg\((\d+)\)=(\d+)$")
def parse_posterior(iterable):
    """
    a generator.
    yields tuples (index, label, prob)

    index: (int) frame index
    label: (int) segment label
    prob: (float) prob value
    """
    # ignores non-matching lines
    for line in iterable:
        m_posterior_entry = re_posterior_entry.match(line.rstrip())

        if m_posterior_entry:
            group = m_posterior_entry.group
            yield (int(group(2)), int(group(3)), float(group(1)))

def read_posterior(infile, num_frames, num_labels):
    """
    returns an array (num_frames, num_labels)
    """
    # XXX: should these be single precision?
    res = zeros((num_frames, num_labels))

    for frame_index, label, prob in parse_posterior(infile):
        if label >= num_labels:
            raise ValueError("saw label %s but num_labels is only %s"
                             % (label, num_labels))

        res[frame_index, label] = prob

    return res


def make_filelistpath(dirpath, ext):
    return dirpath / extjoin(ext, EXT_LIST)

def main(args=sys.argv[1:]):
    pass

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
calc_distance: DESCRIPTION
"""

__version__ = "$Revision: 197 $"

# Copyright 2008 Michael M. Hoffman <mmh1@washington.edu>

from collections import defaultdict
import sys

from numpy import array, dstack, ix_, unique, where
from tabdelim import DictReader
from tables import openFile

from ._util import fill_array, find_segment_starts

FIELDNAMES = ["chrom", "chromStart", "chromEnd"]

def load_bedfiles(filenames):
    res = defaultdict(list)

    for filename in filenames:
        with open(filename) as bedfile:
            reader = DictReader(bedfile, FIELDNAMES)

            for row in reader:
                res[row["chrom"]].append((int(row["chromStart"]),
                                          int(row["chromEnd"])))

    return dict((key, array(value)) for (key, value) in res.iteritems())

def calc_distance(h5filenames, bedfilenames):
    sbjct_all = load_bedfiles(bedfilenames)

    for h5filename in h5filenames:
        with openFile(h5filename) as h5file:
            root = h5file.root
            attrs = root._v_attrs
            chrom = attrs.chrom

            try:
                sbjct = sbjct_all[chrom]
            except KeyError:
                continue

            supercontig_start = attrs.start

            identify = root.identify

            start_pos, labels = find_segment_starts(identify.read())

            seg_labels = unique(labels)

            for seg_label in seg_labels:
                # XXX: duplicative of run.py:print_segment_summary_stats()
                where_seg, = where(labels == seg_label)

                query = start_pos.take([where_seg, where_seg+1])
                query.shape = tuple(reversed(query.shape))
                query += supercontig_start

                # this is like the cross product, but it is a
                # subtraction instead
                sbjct_ix, query_ix = ix_(sbjct.ravel(), query.ravel())
                distances = sbjct_ix - query_ix

                # repeating block of
                # [[sbjct_start-query_start sbjct_start-query_end]
                #  [sbjct_end-query_start   sbjct_end-query_end]]
                sbjct_start_minus_query_end = distances[::2, 1::2]
                sbjct_end_minus_query_start = distances[1::2, ::2]

                overlap = ((sbjct_start_minus_query_end < 0)
                           & (sbjct_end_minus_query_start > 0))
                which_overlap = overlap.sum(0) > 0

                print >>sys.stderr, ("%s/%s: %s/%s"
                                     % (h5filename, seg_label,
                                        which_overlap.sum(), len(query)))

                # compress number of sbjcts to 1
                closest_distances = abs(distances).min(0)

                # reshape so major axis is number of queries
                closest_distances.shape = (len(query), 2)
                closest_distances[which_overlap] = 0

                # compress minor axis (coordinate start or end)
                closest_distances = closest_distances.min(1)

                for closest_distance in closest_distances:
                    print "\t".join(map(str, [seg_label, closest_distance]))

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... H5FILE..."
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)
    parser.add_option("-b", "--bed", action="append", default=[],
                      metavar="BEDFILE", help="compare with BEDFILE")

    options, args = parser.parse_args(args)

    if not len(args) >= 1:
        parser.print_usage()
        sys.exit(1)

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return calc_distance(args, options.bed)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
task: wraps a GMTK subtask to reduce size of output
"""

__version__ = "$Revision: 5777 $"

# Copyright 2009, 2011 Michael M. Hoffman <mmh1@washington.edu>

from errno import ENOENT
from os import extsep, fdopen
import re
import sys
from tempfile import gettempdir, mkstemp

from genomedata import Genome
from numpy import array, zeros, empty, where, diff, r_
from path import path

from ._util import (BED_SCORE, BED_STRAND, ceildiv, DTYPE_IDENTIFY, EXT_FLOAT,
                    EXT_INT, EXT_LIST, find_segment_starts, get_label_color,
                    _make_continuous_cells, POSTERIOR_PROG,
                    POSTERIOR_SCALE_FACTOR, read_posterior,
                    _save_observations_window, VITERBI_PROG)

MSG_SUCCESS = "____ PROGRAM ENDED SUCCESSFULLY WITH STATUS 0 AT"


SCORE_MIN = 100
SCORE_MAX = 1000

TEMP_DIRPATH = path(gettempdir())

EXT_OPTIONS = {}
EXT_OPTIONS[EXT_INT] = "-of1"
EXT_OPTIONS[EXT_FLOAT] = "-of2"

def make_track_indexes(text):
    return array(map(int, text.split(",")))

re_seg = re.compile(r"^seg\((\d+)\)=(\d+)$")
def parse_viterbi(lines):
    """
    returns: numpy.ndarray of size (num_frames,), type DTYPE_IDENTIFY
    """
    lines = iter(lines)

    # Segment 0, after Island[...]
    assert lines.next().startswith("Segment ")

    # ========
    assert lines.next().startswith("========")

    # Segment 0, number of frames = 1001, viteri-score = -6998.363710
    line = lines.next()
    assert line.startswith("Segment ")

    num_frames_text = line.split(", ")[1].partition(" = ")
    assert num_frames_text[0] == "number of frames"
    assert num_frames_text[1] == " = "

    num_frames = int(num_frames_text[2])

    # Printing random variables from (P,C,E)=(1,999,0) partitions
    line = lines.next()
    assert line.startswith("Printing random variables from (P,C,E)")

    res = zeros(num_frames, DTYPE_IDENTIFY)

    for line in lines:
        # Ptn-0 P': seg(0)=24,seg(1)=24
        if line.startswith(MSG_SUCCESS):
            return res

        assert line.startswith("Ptn-")

        values = line.rpartition(": ")[2]

        for pair in values.split(","):
            match = re_seg.match(pair)
            if not match:
                continue

            index = int(match.group(1))
            val = int(match.group(2))

            res[index] = val

    # shouldn't get to this point
    raise ValueError("%s did not complete successfully" % VITERBI_PROG.prog)

# num_cols is for use by genomedata_count_condition
# XXX: should move this function somewhere else
def write_bed(outfile, start_pos, labels, coord, resolution, num_labels,
              num_cols=None):
    """
    start_pos is an array
    """
    (chrom, region_start, region_end) = coord

    start_pos = region_start + (start_pos * resolution)

    # correct last position which may not be a full sample beyond
    if __debug__:
        remainder = (region_end - region_start) % resolution
        if remainder == 0:
            remainder = resolution
        assert region_end == start_pos[-1] - resolution + remainder
    start_pos[-1] = region_end

    # XXX: why is score_step never used? should this line be removed?
    score_step = (SCORE_MAX - SCORE_MIN) / (num_labels - 1)

    zipper = zip(start_pos[:-1], start_pos[1:], labels)

    # this is easily concatenated since it has no context
    for seg_start, seg_end, seg_label in zipper:
        name = str(seg_label)

        chrom_start = str(seg_start)
        chrom_end = str(seg_end)
        item_rgb = get_label_color(seg_label)

        row = [chrom, chrom_start, chrom_end, name, BED_SCORE, BED_STRAND,
               chrom_start, chrom_end, item_rgb][:num_cols]

        print >>outfile, "\t".join(row)

    # assert that the whole region is mapped
    # seg_end here means the last seg_end in the loop
    assert seg_end == region_end

def save_bed(outfilename, *args, **kwargs):
    with open(outfilename, "w") as outfile:
        write_bed(outfile, *args, **kwargs)

def read_posterior_save_bed(coord, resolution,  outfilename_tmpl, num_labels,
                            infile):
    (chrom, start, end) = coord
    num_frames = ceildiv(end - start, resolution)
    probs = read_posterior(infile, num_frames, num_labels)
    probs_rounded = empty(probs.shape, int)

    outfilenames = []
    for label_index in xrange(num_labels):
        outfilenames.append(outfilename_tmpl % label_index)

    # scale, round, and cast to int
    (probs * POSTERIOR_SCALE_FACTOR).round(out = probs_rounded)

    # print array columns as text to each outfile
    zipper = zip(outfilenames, probs_rounded.T, xrange(num_labels))
    for outfilename, probs_rounded_label, label_index in zipper:
        # run-length encoding on the probs_rounded_label

        outfile = open(outfilename, "w")
        pos, = where(diff(probs_rounded_label) != 0)
        pos = r_[start, pos[:]+start+1, end]

        for bed_start, bed_end in zip(pos[:-1], pos[1:]):
            chrom_start = str(bed_start)
            chrom_end = str(bed_end)
            value = str(probs_rounded_label[bed_start-start])

            row = [chrom, chrom_start, chrom_end, value]
            print >>outfile, "\t".join(row)

def load_posterior_save_bed(coord, resolution, outfilename, num_labels,
                            infilename):
    with open(infilename) as infile:
        read_posterior_save_bed(coord, resolution, outfilename,
                                int(num_labels), infile)

def parse_viterbi_save_bed(coord, resolution, viterbi_lines, bed_filename, num_labels):
    data = parse_viterbi(viterbi_lines)

    start_pos, labels = find_segment_starts(data)

    save_bed(bed_filename, start_pos, labels, coord, resolution, int(num_labels))

def load_viterbi_save_bed(coord, resolution, outfilename, num_labels, infilename):
    with open(infilename) as infile:
        lines = infile.readlines()

    return parse_viterbi_save_bed(coord, resolution, lines, outfilename,
                                  num_labels)

def replace_args_filelistname(args, temp_filepaths, ext):
    option = EXT_OPTIONS[ext]
    filelistname_index = args.index(option) + 1
    fd, filelistname = mkstemp(EXT_LIST + extsep, extsep + ext)
    filelistpath = path(filelistname)

    # side-effect on args, temp_filepaths
    args[filelistname_index] = filelistname
    temp_filepaths.append(filelistpath)

    return fd

def print_to_fd(fd, line):
    with fdopen(fd, "w") as outfile:
        print >>outfile, line

def run_posterior_save_bed(coord, resolution, outfilename, num_labels,
                           genomedata_dirname, float_filename, int_filename,
                           distribution, track_indexes_text, *args):
    # XXX: this whole function is duplicative of run_viterbi_save_bed and needs to be reduced
    # convert from tuple
    args = list(args)

    # a 2,000,000-frame output file is only 84 MiB so it is okay to
    # read the whole thing into memory
    (chrom, start, end) = coord
    track_indexes = make_track_indexes(track_indexes_text)

    float_filepath = TEMP_DIRPATH / float_filename
    int_filepath = TEMP_DIRPATH / int_filename
    temp_filepaths = [float_filepath, int_filepath]

    # XXX: should do something to ensure of1 matches with int, of2 with float
    int_filelistfd = replace_args_filelistname(args, temp_filepaths, EXT_INT)
    float_filelistfd = replace_args_filelistname(args, temp_filepaths,
                                                 EXT_FLOAT)
    with Genome(genomedata_dirname) as genome:
        supercontigs = genome[chrom].supercontigs[start:end]
        assert len(supercontigs) == 1
        supercontig = supercontigs[0]

        continuous_cells = _make_continuous_cells(supercontig, start, end,
                                                  track_indexes)
    try:
        print_to_fd(float_filelistfd, float_filename)
        print_to_fd(int_filelistfd, int_filename)

        _save_observations_window(float_filename, int_filename,
                                  continuous_cells, resolution, distribution)

        # XXXopt: does this actually free the memory? or do we need to
        # use a subprocess to do the loading?

        # remove from memory
        del continuous_cells

        output = POSTERIOR_PROG.getoutput(*args)
    finally:
        for filepath in temp_filepaths:
            # don't raise a nested exception if the file was never created
            try:
                filepath.remove()
            except OSError, err:
                if err.errno == ENOENT:
                    pass

    lines = output.splitlines()
    return read_posterior_save_bed(coord, resolution,  outfilename,
                                   int(num_labels), lines)


def run_viterbi_save_bed(coord, resolution, outfilename, num_labels,
                         genomedata_dirname, float_filename, int_filename,
                         distribution, track_indexes_text, *args):
    # convert from tuple
    args = list(args)
    # a 2,000,000-frame output file is only 84 MiB so it is okay to
    # read the whole thing into memory

    (chrom, start, end) = coord

    track_indexes = make_track_indexes(track_indexes_text)

    float_filepath = TEMP_DIRPATH / float_filename
    int_filepath = TEMP_DIRPATH / int_filename

    temp_filepaths = [float_filepath, int_filepath]

    # XXX: should do something to ensure of1 matches with int, of2 with float
    int_filelistfd = replace_args_filelistname(args, temp_filepaths, EXT_INT)
    float_filelistfd = replace_args_filelistname(args, temp_filepaths,
                                                   EXT_FLOAT)

    with Genome(genomedata_dirname) as genome:
        supercontigs = genome[chrom].supercontigs[start:end]
        assert len(supercontigs) == 1
        supercontig = supercontigs[0]

        continuous_cells = _make_continuous_cells(supercontig, start, end,
                                                  track_indexes)

    try:
        print_to_fd(float_filelistfd, float_filename)
        print_to_fd(int_filelistfd, int_filename)

        _save_observations_window(float_filename, int_filename,
                                  continuous_cells, resolution, distribution)

        # XXXopt: does this work? or do we need to use a subprocess to
        # do the loading?
        # remove from memory
        del continuous_cells

        output = VITERBI_PROG.getoutput(*args)
    finally:
        for filepath in temp_filepaths:
            # don't raise a nested exception if the file was never created
            try:
                filepath.remove()
            except OSError, err:
                if err.errno == ENOENT:
                    pass

    lines = output.splitlines()

    return parse_viterbi_save_bed(coord, resolution, lines, outfilename,
                                  num_labels)

TASKS = {("run", "viterbi"): run_viterbi_save_bed,
         ("load", "viterbi"): load_viterbi_save_bed,
         ("run", "posterior"): run_posterior_save_bed,
         ("load", "posterior"): load_posterior_save_bed}

def task(verb, kind, outfilename, chrom, start, end, resolution, *args):
    start = int(start)
    end = int(end)
    resolution = int(resolution)

    TASKS[verb, kind]((chrom, start, end), resolution, outfilename, *args)

def main(args=sys.argv[1:]):
    if len(args) < 7:
        print >>sys.stderr, \
            "args: VERB KIND OUTFILE CHROM START END RESOLUTION [ARGS...]"
        sys.exit(2)

    return task(*args)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division

__version__ = "$Revision: 5152 $"

# Copyright 2009, 2011 Michael M. Hoffman <mmh1@washington.edu>

from math import ceil
from resource import getrlimit, RLIMIT_STACK
import sys

from .._util import MB
from .common import _JobTemplateFactory, make_native_spec

# qsub -w: switches off job validation
NATIVE_SPEC_DEFAULT = dict(w="n")

try:
    STACK_LIMIT = min(num for num in getrlimit(RLIMIT_STACK)
                      if num > 0)
except ValueError:
    STACK_LIMIT = 10*MB

class JobTemplateFactory(_JobTemplateFactory):
    def make_res_req(self, mem_usage, tmp_usage):
        # tmp_usage unused on SGE

        # By default the stack limit is so high that there is an
        # automatic crash when you also set a virtual memory limit and
        # then start new pthreads. Setting h_stack is necessary if you
        # are setting h_vmem.
        return [make_single_res_req("mem_requested", mem_usage),
                make_single_res_req("h_vmem", self.mem_limit),
                make_single_res_req("h_stack", STACK_LIMIT)]

    def make_native_spec(self):
        # qsub -l: resource requirement
        res_spec = make_native_spec(l=self.res_req)

        res = " ".join([self.native_spec,
                        make_native_spec(**NATIVE_SPEC_DEFAULT),
                        res_spec])

        return res

def make_single_res_req(name, mem):
    # round up to the next mebibyte
    return "%s=%dM" % (name, ceil(mem / MB))

def main(args=sys.argv[1:]):
    pass

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
sge_setup: setup mem_requested on each node
"""

__version__ = "$Revision: 4295 $"

# Copyright 2010 Michael M. Hoffman <mmh1@uw.edu>

import sys
from tempfile import NamedTemporaryFile

from optbuild import OptionBuilder_ShortOptWithSpace
# from path import path

QCONF_PROG = OptionBuilder_ShortOptWithSpace("qconf")
QSTAT_PROG = OptionBuilder_ShortOptWithSpace("qstat")

MEM_TOTAL_STR = "hl:mem_total="
HOSTNAME_STR = "qf:hostname="

USERNAME = "fakeuser"

OUTPUT_RECORD_SEPARATOR = \
    "----------------------------------------------------------------------------\n"

def sge_setup():
    tempfile = NamedTemporaryFile("w", suffix=".txt", prefix="qconf.")

    prior_complex_text = QCONF_PROG.getoutput(sc=True)
    print >>tempfile, prior_complex_text
    print >>tempfile, "mem_requested\tmr\tMEMORY\t<=\tYES\tYES\t0\t10"
    # XXX: after Python 2.6, use tempfile.close() with NamedTemporaryFile(delete=False)
    tempfile.flush()

    QCONF_PROG(Mc=tempfile.name)

    # path(tempfile.name).unlink()

    stat_texts_text = QSTAT_PROG.getoutput(F="hostname,mem_total", u=USERNAME)
    stat_texts = stat_texts_text.split(OUTPUT_RECORD_SEPARATOR)

    mem_totals = {}

    # skip header
    for stat_text in stat_texts[1:]:
        lines_dict = {}

        lines = stat_text.splitlines()
        for line in lines:
            key, equals, value = line.strip().partition("=")
            if equals == "=":
                lines_dict[key] = value

        # you might get a hostname reported multiple times for each queue
        # we'll overwrite instead of checking for equality
        mem_totals[lines_dict["qf:hostname"]] = lines_dict["hl:mem_total"]

    for hostname, mem_total in mem_totals.iteritems():
        QCONF_PROG("-mattr", "exechost", "complex_values",
                   "mem_requested=%s" % mem_total, hostname)

    # extra newline at beginning to space from qconf messages
    print >>sys.stderr, """
===========================================================================
The mem_requested resource is now configured on your cluster. While
Segway uses this resource to control its own jobs, we recommend
setting a default allocation for all other jobs by editing the default
SGE request. Otherwise they will not request any memory unless
explicitly specified, which can lead to conflicts with
mem_requested-aware applications. For example, you can edit the file
$SGE_ROOT/$SGE_CELL/common/sge_request to add this line:

-l mem_requested=2G

This will request 2 GiB of RAM for an otherwise unspecfied job, which
is usually a good idea on a cluster where hosts have, say, 8 cores and
16 GiB of RAM.

It's also a good idea to verify the configuration by running:

qhost -F mem_requested
===========================================================================
"""

# XXX: a dry-run option would be good

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]..."
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    options, args = parser.parse_args(args)

    if not len(args) == 0:
        parser.error("incorrect number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return sge_setup(*args)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
h5values: prints finite values in a column of a number of HDF5 chromosome
files
"""

__version__ = "$Revision: 53 $"

# Copyright 2008 Michael M. Hoffman <mmh1@washington.edu>

import sys

from numpy import isfinite
from tables import openFile

from ._util import walk_continuous_supercontigs

# XXX: duplicative of h5histogram.py
def h5values(col_index, filenames):
    for filename in filenames:
        with openFile(filename) as h5file:
            print >>sys.stderr, filename
            supercontig_walker = walk_continuous_supercontigs(h5file)
            for supercontig, continuous in supercontig_walker:
                col = continuous[:, col_index]
                col_finite = col[isfinite(col)]
                print "\n".join(str(num) for num in col_finite)

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... FILE..."
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)
    parser.add_option("-c", "--col", type=int, metavar="COL",
                      default=0, help="write values in column COL")

    options, args = parser.parse_args(args)

    if not len(args) >= 1:
        parser.print_usage()
        sys.exit(1)

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return h5values(options.col, args)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division

__version__ = "$Revision: 5698 $"

# common stuff: needs to be in a different file from __init__ because
# it is imported by lsf.py and sge.py
# Copyright 2009, 2011 Michael M. Hoffman <mmh1@washington.edu>

import sys

from optbuild import Mixin_NoConvertUnderscore, OptionBuilder_ShortOptWithSpace

from .._util import data_filename, KB, MB

NATIVE_SPEC_PROG = (Mixin_NoConvertUnderscore
                    + OptionBuilder_ShortOptWithSpace)() # do not run

# guard space to prevent going over mem_requested allocation
MEM_GUARD = 10*MB

BASH_CMD = "bash"
RES_WRAPPER = "segway-wrapper.sh"

MSG_EDGE = """
Ran through entire memory usage progression reached without success.
This is probably not related to memory usage, however. Check for errors in %s.
See the Troubleshooting section of the Segway documentation."""

class JobError(RuntimeError):
    pass

class _JobTemplateFactory(object):
    # this might be overridden by a subclass
    set_template_output_error = True

    def __init__(self, template, tmp_usage, mem_usage_progression,
                 output_filename, error_filename):
        self.args = template.args
        self.native_spec = template.nativeSpecification
        self.mem_usage_progression = mem_usage_progression

        # set here so that it can be overridden for LSF
        self.output_filename = output_filename
        self.error_filename = error_filename

        if self.set_template_output_error:
            template.outputPath = ":" + output_filename
            template.errorPath = ":" + error_filename

        self.template = template
        self.tmp_usage = tmp_usage

    def __call__(self, trial_index):
        """
        returns a job template with the attributes set for the next
        memory progression step
        """
        res = self.template

        try:
            mem_usage = self.mem_usage_progression[trial_index]
        except IndexError:
            raise RuntimeError(MSG_EDGE % self.error_filename)

        self.mem_limit = int(mem_usage)
        self.res_req = self.make_res_req(mem_usage, self.tmp_usage)

        res.args = self.make_args()
        res.nativeSpecification = self.make_native_spec()

        return res

    def make_res_req(self, mem_usage, tmp_usage):
        # pure virtual function
        raise NotImplementedError

    def make_args(self):
        """
        wrap args with segway-wrapper.sh
        """
        # ulimit args are in kibibytes
        mem_limit_kb = str(calc_mem_limit(self.mem_limit) // KB)
        wrapper_cmdline = [BASH_CMD, data_filename(RES_WRAPPER), mem_limit_kb]

        return wrapper_cmdline + self.args

    def make_native_spec(self):
        # pure virtual function
        raise NotImplementedError

def make_native_spec(*args, **kwargs):
    return " ".join(NATIVE_SPEC_PROG.build_args(args=args, options=kwargs))

def calc_mem_limit(mem_usage):
    """
    amount of memory allowed to be used (less than that requested)
    """
    return int(mem_usage - MEM_GUARD)

def main(args=sys.argv[1:]):
    pass

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division

"""winner.py: pick winning paramters when training run is cut short
"""

__version__ = "$Revision: 5716 $"

## Copyright 2011 Michael M. Hoffman <mmh1@uw.edu>

from operator import itemgetter
from os import extsep
from re import compile as re_compile, escape
import sys

from path import path

from ._util import (extjoin, EXT_MASTER, EXT_PARAMS, EXT_TAB, PREFIX_INPUT,
                    PREFIX_LIKELIHOOD, PREFIX_PARAMS, SUBDIRNAME_LOG, SUBDIRNAME_PARAMS)

PAT_LIKELIHOOD = extjoin(PREFIX_LIKELIHOOD, "*", EXT_TAB)

def extjoin_escape(*args):
    return escape(extsep).join(args)

re_likelihood = re_compile(extjoin_escape(escape(PREFIX_LIKELIHOOD), "(.*)",
                                          escape(EXT_TAB)))
def get_likelihood_index(filepath):
    """
    returns a str
    """
    return re_likelihood.match(filepath.name).group(1)

def load_likelihood(filename):
    with open(filename) as infile:
        # get last line
        for line in infile:
            pass

    return float(line)

def enumerate_likelihoods(dirpath):
    log_dirpath = dirpath / SUBDIRNAME_LOG
    for filepath in log_dirpath.files(PAT_LIKELIHOOD):
        yield get_likelihood_index(filepath), load_likelihood(filepath)

def get_winning_instance(dirpath):
    return sorted(enumerate_likelihoods(dirpath), key=itemgetter(1))[-1][0]

def enumerate_params_filenames(dirpath, instance):
    pattern = extjoin(PREFIX_PARAMS, instance, EXT_PARAMS, "*")

    for filename in dirpath.files(pattern):
        yield int(filename.rpartition(extsep)[2]), filename

def get_last_params_filename(dirpath, instance):
    return sorted(enumerate_params_filenames(dirpath, instance))[-1][-1]

def get_input_master_filename(dirpath, instance):
    return dirpath / extjoin(PREFIX_INPUT, instance, EXT_MASTER)

def print_and_copy(flag, getter, dirpath, instance, final_basename, copy, clobber):
    if flag:
        srcpath = getter(dirpath, instance)
        print srcpath

        dstpath = dirpath / final_basename

        if clobber or not dstpath.exists():
            srcpath.copy2(dstpath)

def winner(dirname, params=True, input_master=True, copy=False, clobber=False):
    dirpath = path(dirname)
    winning_instance = get_winning_instance(dirpath)

    params_dirpath = dirpath / SUBDIRNAME_PARAMS
    print_and_copy(input_master, get_input_master_filename, params_dirpath,
                   winning_instance, "input.master", copy, clobber)
    print_and_copy(params, get_last_params_filename, params_dirpath,
                   winning_instance, "params.params", copy, clobber)

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... DIR"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    parser.add_option("-i", "--input-master", action="store_true",
                      help="print input master file name")
    parser.add_option("-p", "--params", action="store_true",
                      help="print parameters file name")
    parser.add_option("-c", "--copy", action="store_true",
                      help="copy files to final winning file locations")
    parser.add_option("--clobber", action="store_true",
                      help="overwrite existing files")

    options, args = parser.parse_args(args)

    if not len(args) == 1:
        parser.error("incorrect number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    params = options.params
    input_master = options.input_master

    # if neither option is used, both are enabled
    if params is None and input_master is None:
        params = True
        input_master = True

    return winner(args[0], params=params, input_master=input_master,
                  copy=options.copy, clobber=options.clobber)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
gtf2bed: DESCRIPTION
"""

__version__ = "$Revision: 199 $"

# Copyright 2008 Michael M. Hoffman <mmh1@washington.edu>

import sys

from tabdelim import DictReader, DictWriter

# http://genome.ucsc.edu/FAQ/FAQformat
INFIELDNAMES = ["seqname", "source", "feature", "start", "end", "score",
                "strand", "frame", "attributes"]

OUTFIELDNAMES = ["chrom", "chromStart", "chromEnd"]

def read_gtf_write_bed(reader, writer, source, feature, only_5prime):
    for row in reader:
        if source is not None and row["source"] != source:
            continue

        if feature is not None and row["feature"] != feature:
            continue

        chrom = "chr" + row["seqname"]
        chromStart = int(row["start"]) - 1
        chromEnd = int(row["end"])

        if only_5prime:
            strand = row["strand"]

            if strand == "+":
                chromEnd = chromStart + 1
            elif strand == "-":
                chromStart = chromEnd - 1
            else:
                raise ValueError("can't get 5' end of feature without strand")

        writer.writerow(locals())

def gtf2bed(filenames, source=None, feature=None, only_5prime=False):
    writer = DictWriter(sys.stdout, OUTFIELDNAMES,
                        extrasaction="ignore", header=False)

    if not filenames:
        filenames = ["-"]

    for filename in filenames:
        if filename == "-":
            infile = sys.stdin
        else:
            infile = open(filename)

        with infile:
            reader = DictReader(infile, INFIELDNAMES)

            read_gtf_write_bed(reader, writer, source, feature, only_5prime)

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... [FILE...]"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    # XXX: group: limit
    parser.add_option("-s", "--source", metavar="SOURCE",
                      help="limit to source SOURCE")
    parser.add_option("-f", "--feature", metavar="FEATURE",
                      help="limit to feature FEATURE")

    # XXX: group: options
    parser.add_option("-5", "--only-5prime", action="store_true",
                      help="reduce features to their 5'-most coordinate")

    options, args = parser.parse_args(args)

    if not len(args) >= 0:
        parser.print_usage()
        sys.exit(1)

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return gtf2bed(args, options.source, options.feature, options.only_5prime)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
h5histogram: prints histogram

XXX a lot of this is copied and updated in the external validation tool
"""

__version__ = "$Revision: 259 $"

# Copyright 2008 Michael M. Hoffman <mmh1@washington.edu>

import sys

from collections import defaultdict
from functools import partial
from numpy import (array, concatenate, histogram, iinfo, isfinite, ndarray,
                   NINF, PINF, zeros)
from tables import openFile

from ._util import (DTYPE_IDENTIFY, fill_array,
                    get_col_index as _get_col_index, iter_chroms_coords,
                    load_coords, walk_continuous_supercontigs)

FIELDNAMES = ["lower_edge", "count"]

IINFO_IDENTIFY = iinfo(DTYPE_IDENTIFY)
MAX_IDENTIFY = IINFO_IDENTIFY.max # sentinel for padding

def get_col_index(chromosome, trackname):
    if trackname is None:
        return 0
    else:
        return _get_col_index(chromosome, trackname)

def calc_range(trackname, filenames):
    # not limited to include_coords, so scale is always the same
    minimum = PINF
    maximum = NINF

    for filename in filenames:
        with openFile(filename) as chromosome:
            col_index = get_col_index(chromosome, trackname)

            attrs = chromosome.root._v_attrs
            minimum = min(minimum, attrs.mins[col_index])
            maximum = max(maximum, attrs.maxs[col_index])

    return minimum, maximum

def calc_histogram(trackname, filenames, data_range, num_bins, include_coords,
                   include_identify_dict, identify_label):
    histogram_custom = partial(histogram, bins=num_bins, range=data_range,
                               new=True)

    hist, edges = histogram_custom(array([]))
    chrom_iterator = iter_chroms_coords(filenames, include_coords)
    for chrom, filename, chromosome, chr_include_coords in chrom_iterator:
        assert not chromosome.root._v_attrs.dirty

        col_index = get_col_index(chromosome, trackname)

        supercontig_walker = walk_continuous_supercontigs(chromosome)
        for supercontig, continuous in supercontig_walker:
            supercontig_attrs = supercontig._v_attrs
            supercontig_start = supercontig_attrs.start
            supercontig_end  = supercontig_attrs.end

            if include_coords:
                # adjust coords
                supercontig_include_coords = (chr_include_coords
                                              - supercontig_start)

                # set all negative coords to 0
                supercontig_include_coords[supercontig_include_coords < 0] = 0
            else:
                # slice(None, None) means the whole sequence
                supercontig_include_coords = [[None, None]]

            for coords in supercontig_include_coords:
                if coords[0] >= len(continuous):
                    continue

                row_slice = slice(*coords)

                col = continuous[row_slice, col_index]

                if include_identify_dict:
                    include_identify_chunks = include_identify_dict[chrom]

                    col_bitmap = zeros(col.shape, bool)

                    coords_start, coords_end = coords
                    if coords_start is None:
                        coords_start = 0
                    if coords_end is None:
                        coords_end = supercontig_end

                    for chunk in include_identify_chunks:
                        chunk_attrs = chunk.root._v_attrs
                        chunk_start = chunk_attrs.start - supercontig_start
                        chunk_end = chunk_attrs.end - supercontig_start

                        chunk_identify = chunk.root.identify
                        assert chunk_identify.atom.dtype == DTYPE_IDENTIFY

                        if (chunk_start >= coords_end
                            or chunk_end <= coords_start):
                            continue

                        chunk_identify_array = None

                        chunk_start_offset = coords_start - chunk_start
                        if chunk_start_offset > 0:
                            chunk_identify_array = \
                                chunk_identify[chunk_start_offset:]
                        elif chunk_start_offset < 0:
                            padding_shape = (-chunk_start_offset,)
                            padding = fill_array(MAX_IDENTIFY, padding_shape,
                                                 DTYPE_IDENTIFY)
                            padded_list = [padding, chunk_identify.read()]

                            chunk_identify_array = concatenate(padded_list)

                        # if there is not enough padding at end,
                        # things will work. Need to correct if there
                        # is too much
                        if chunk_end > coords_end:
                            # -1 if it is one longer, etc.
                            chunk_end_offset = chunk_end-coords_end
                            chunk_identify_array = \
                                chunk_identify[:chunk_end_offset]

                        if chunk_identify_array is None:
                            chunk_identify_array = chunk_identify.read()

                        assert isinstance(chunk_identify_array, ndarray)

                        assert (len(chunk_identify_array)
                                <= coords_end - coords_start)

                        loc_true = chunk_identify_array == identify_label
                        col_bitmap[loc_true] = True

                    col = col[col_bitmap]

                col_finite = col[isfinite(col)]

                # if it has at least one row (it isn't truncated
                # away by the include_coords)
                if col_finite.shape[0]:
                    if coords[0] is not None:
                        coords_tuple = tuple(coords + supercontig_start)
                        print >>sys.stderr, " (%s, %s)" % coords_tuple

                    hist_supercontig, edges_supercontig = \
                        histogram_custom(col_finite)

                    assert (edges_supercontig == edges).all()
                    hist += hist_supercontig

    return hist, edges

def print_histogram(hist, edges):
    for row in zip(edges, hist.tolist() + ["NA"]):
        print "\t".join(map(str, row))

def load_include_identify(filelistname):
    if filelistname is None:
        return {}

    res = defaultdict(list)

    with open(filelistname) as filelist:
        for line in filelist:
            filename = line.rstrip()

            # XXX: these never get closed
            identify = openFile(filename)
            chrom = identify.root._v_attrs.chrom
            res[chrom].append(identify)

    return res

def h5histogram(trackname, filenames, num_bins, include_coords_filename=None,
                include_identify_filelistname=None, identify_label=1):
    print "\t".join(FIELDNAMES)

    # two passes to avoid running out of memory
    data_range = calc_range(trackname, filenames)

    include_coords = load_coords(include_coords_filename)
    include_identify_dict = \
        load_include_identify(include_identify_filelistname)

    try:
        hist, edges = calc_histogram(trackname, filenames, data_range,
                                     num_bins, include_coords,
                                     include_identify_dict, identify_label)
    finally:
        for include_identify_h5files in include_identify_dict.itervalues():
            for include_identify_h5file in include_identify_h5files:
                include_identify_h5file.close()

    print_histogram(hist, edges)

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... FILE..."
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)
    # this is a 0-based file (I know because ENm008 starts at position 0)
    parser.add_option("--include-coords", metavar="FILE",
                      help="limit to genomic coordinates in FILE")

    parser.add_option("--include-identify", metavar="FILELIST",
                      help="limit to label identified in files in FILELIST")

    parser.add_option("--identify-label", metavar="LABEL", default=1, type=int,
                      help="limit to LABEL in a list of specified identify"
                      " files")

    parser.add_option("-c", "--col", metavar="COL",
                      help="write values in column COL (default first column)")

    parser.add_option("-b", "--num-bins", metavar="BINS", type=int,
                      default=100, help="use BINS bins")

    options, args = parser.parse_args(args)

    if not len(args) >= 1:
        parser.print_usage()
        sys.exit(1)

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return h5histogram(options.col, args, options.num_bins,
                       options.include_coords, options.include_identify,
                       options.identify_label)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division

"""gethelp.py: get help from a command-line script
"""

__version__ = "$Revision: 5384 $"

## Copyright 2011 Michael M. Hoffman <mmh1@uw.edu>

from ConfigParser import RawConfigParser
from cStringIO import StringIO
import sys

sys.path.insert(0, "..")
from setup import entry_points

def gethelp(scriptname):
    config = RawConfigParser()
    config.readfp(StringIO(entry_points))

    entry = config.get("console_scripts", scriptname).split()[0]
    module_name, _, func_name = entry.partition(":")

    # __import__(module_name) usually returns the top-level package module only
    # so get our module out of sys.modules instead
    __import__(module_name)
    module = sys.modules[module_name]

    sys.argv[0] = scriptname
    getattr(module, func_name)(["--help"])

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTION]... SCRIPTNAME"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    options, args = parser.parse_args(args)

    if not len(args) == 1:
        parser.error("incorrect number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_options(args)

    return gethelp(*args)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

"""segway: a way to segment the genome

Segway is a tool for easy pattern discovery and identification in
functional genomics data.
"""

__version__ = "1.1.0"

# Copyright 2008-2011 Michael M. Hoffman <mmh1@washington.edu>

import sys

# required for from __future__ import division, with_statement
assert sys.version_info >= (2, 5, 1)

from ez_setup import use_setuptools
use_setuptools()

from setuptools import find_packages, setup

doclines = __doc__.splitlines()
name, short_description = doclines[0].split(": ")
long_description = "\n".join(doclines[2:])

url = "http://noble.gs.washington.edu/proj/%s/" % name.lower()
download_url = "%ssrc/%s-%s.tar.gz" % (url, name, __version__)

classifiers = ["Natural Language :: English",
               "Programming Language :: Python"]

# XXX: parallel should be an option, after the code is written
# correctly to fallback when it's not there
entry_points = """
[console_scripts]
segway = segway.run:main [parallel]
segway-calc-distance = segway.calc_distance:main
segway-task = segway.task:main
segway-layer = segway.layer:main
segway-winner = segway.winner:main

h5histogram = segway.h5histogram:main
h5values = segway.h5values:main

gtf2bed = segway.gtf2bed:main
"""

# XXX: warn: make sure you have LDFLAGS unset if you are building numpy

# need optbuild>0.1.5 for Mixin_UseFullProgPath
# need tables>2.04 (>=r3761) because there is a CArray fill bug until then

install_requires = ["genomedata>0.1.5", "textinput", "optbuild>=0.1.10",
                    "optplus>0.1.0", "tables>2.0.4", "numpy", "forked-path",
                    "colorbrewer"]

# XXX: ask if there is a way to specify this at the command-line
extras_require = dict(parallel=["drmaa>=0.4a3"])

if __name__ == "__main__":
    setup(name=name,
          version=__version__,
          description=short_description,
          author="Michael Hoffman",
          author_email="mmh1@uw.edu",
          url=url,
          download_url=download_url,
          classifiers=classifiers,
          long_description=long_description,
          install_requires=install_requires,
          extras_require=extras_require,
          zip_safe=False, # XXX: change back, this is just for better tracebacks

          # XXX: this should be based off of __file__ instead
          packages=find_packages("."),
          include_package_data=True,
          entry_points=entry_points
          )
